@@ -1,16 +1,8 @@
-// Copyright (C) 2014-2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2016 Internet Systems Consortium, Inc. ("ISC")
 //
-// Permission to use, copy, modify, and/or distribute this software for any
-// purpose with or without fee is hereby granted, provided that the above
-// copyright notice and this permission notice appear in all copies.
-//
-// THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
-// REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
-// AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
-// INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
-// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
-// OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
-// PERFORMANCE OF THIS SOFTWARE.
+// This Source Code Form is subject to the terms of the Mozilla Public
+// License, v. 2.0. If a copy of the MPL was not distributed with this
+// file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 #ifndef DHCP6_CLIENT_H
 #define DHCP6_CLIENT_H
@@ -19,9 +11,12 @@
 #include <dhcp/duid.h>
 #include <dhcp/option.h>
 #include <dhcp/option6_client_fqdn.h>
+#include <dhcpsrv/lease.h>
 #include <dhcp6/tests/dhcp6_test_utils.h>
+#include <util/staged_value.h>
 #include <boost/noncopyable.hpp>
 #include <boost/shared_ptr.hpp>
+#include <list>
 #include <set>
 #include <vector>
 
@@ -32,7 +27,7 @@ namespace test {
 /// @brief DHCPv6 client used for unit testing.
 ///
 /// This class implements a DHCPv6 "client" which interoperates with the
-/// @c NakedDhcpv6Srv class. It calls @c NakedDhcpv6Srv::fakeRecive to
+/// @c NakedDhcpv6Srv class. It calls @c NakedDhcpv6Srv::fakeReceive to
 /// deliver client messages to the server for processing. The server places
 /// the response in the @c NakedDhcpv6Srv::fake_sent_ container. The client
 /// pops messages from this container which simulates reception of the
@@ -150,6 +145,27 @@ public:
         Pkt6Ptr response_;
     };
 
+    /// @brief Structure holding information to be placed in client's IA.
+    struct ClientIA {
+        Lease::Type type_;            ///< IA type
+        uint32_t iaid_;               ///< IAID
+        asiolink::IOAddress prefix_;  ///< prefix or address
+        uint8_t prefix_len_;          ///< prefix length
+
+        /// @brief Constructor.
+        ///
+        /// @param type IA type.
+        /// @param iaid IAID.
+        /// @param prefix Address or prefix.
+        /// @param prefix_len Prefix length.
+        ClientIA(const Lease::Type& type, const uint32_t iaid,
+                 const asiolink::IOAddress& prefix,
+                 const uint8_t prefix_len)
+            : type_(type), iaid_(iaid), prefix_(prefix),
+              prefix_len_(prefix_len) {
+        }
+    };
+
     /// @brief Creates a new client.
     ///
     /// This constructor initializes the class members to default values:
@@ -211,12 +227,15 @@ public:
     /// i.e. sends a Solicit to the server and receives Advertise. It doesn't
     /// set the lease configuration in the @c config_.
     ///
+    /// @param always_apply_config Apply received configuration even if the
+    /// Advertise message is received. Default value is false.
+    ///
     /// @throw This function doesn't throw exceptions on its own, but it calls
     /// functions that are not exception safe, so it may throw exceptions if
     /// error occurs.
     ///
     /// @todo Perform sanity checks on returned messages.
-    void doSolicit();
+    void doSolicit(const bool always_apply_config = false);
 
     /// @brief Sends a Renew to the server and receives the Reply.
     ///
@@ -273,13 +292,14 @@ public:
     ///
     /// This function simulates sending the Decline message to the server and
     /// receiving the server's response.
-    void doDecline();
+    /// @param include_address should the address be included?
+    void doDecline(const bool include_address = true);
 
     /// @brief Performs stateless (inf-request / reply) exchange.
     ///
     /// This function generates Information-request message, sends it
     /// to the server and then receives the reply. Contents of the Inf-Request
-    /// are controlled by use_na_, use_pd_, use_client_id_ and use_oro_
+    /// are controlled by client_ias_, use_client_id_ and use_oro_
     /// fields. This method does not process the response in any specific
     /// way, just stores it.
     void doInfRequest();
@@ -343,6 +363,110 @@ public:
     /// @brief Returns leases with zero lifetimes.
     std::vector<Lease6> getLeasesWithZeroLifetime() const;
 
+    /// @brief Returns leases by lease address/prefix.
+    ///
+    /// @param address Leased address.
+    ///
+    /// @return Vector containing leases for the specified address.
+    std::vector<Lease6>
+    getLeasesByAddress(const asiolink::IOAddress& address) const;
+
+    /// @brief Returns leases belonging to specified address range.
+    ///
+    /// @param first Lower bound of the address range.
+    /// @param second Upper bound of the address range.
+    ///
+    /// @return Vector containing leases belonging to specified address range.
+    std::vector<Lease6>
+    getLeasesByAddressRange(const asiolink::IOAddress& first,
+                            const asiolink::IOAddress& second) const;
+
+    /// @brief Returns leases belonging to prefix pool.
+    ///
+    /// @param prefix Prefix of the pool.
+    /// @param prefix_len Prefix length.
+    /// @param delegated_len Delegated prefix length.
+    ///
+    /// @return Vector containing leases belonging to specified prefix pool.
+    std::vector<Lease6>
+    getLeasesByPrefixPool(const asiolink::IOAddress& prefix,
+                          const uint8_t prefix_len,
+                          const uint8_t delegated_len) const;
+
+    /// @brief Checks if client has lease for the specified address.
+    ///
+    /// @param address Address for which lease should be found.
+    ///
+    /// @return true if client has lease for the address, false otherwise.
+    bool hasLeaseForAddress(const asiolink::IOAddress& address) const;
+
+    /// @brief Checks if client has lease for the specified address in the
+    /// IA_NA identified by IAID.
+    ///
+    /// @param address Address for which lease should be found.
+    /// @param iaid IAID of the IA_NA in which the lease is expected.
+    bool hasLeaseForAddress(const asiolink::IOAddress& address,
+                            const IAID& iaid) const;
+
+    /// @brief Checks if client has a lease for an address within range.
+    ///
+    /// @param first Lower bound of the address range.
+    /// @param last Upper bound of the address range.
+    ///
+    /// @return true if client has lease for the address within the range,
+    /// false otherwise.
+    bool hasLeaseForAddressRange(const asiolink::IOAddress& first,
+                                 const asiolink::IOAddress& last) const;
+
+    /// @brief Checks if client has a lease with zero lifetimes for the
+    /// specified address.
+    ///
+    /// @param address Address for which lease should be found.
+    ///
+    /// @return true if client has a lease, false otherwise.
+    bool hasLeaseWithZeroLifetimeForAddress(const asiolink::IOAddress& address) const;
+
+    /// @brief Checks if client has a lease for a prefix.
+    ///
+    /// @param prefix Prefix.
+    /// @param prefix_len Prefix length.
+    ///
+    /// @return true if client has a lease for the specified prefix, false
+    /// otherwise.
+    bool hasLeaseForPrefix(const asiolink::IOAddress& prefix,
+                           const uint8_t prefix_len) const;
+
+    /// @brief Checks if client as a lease for prefix in the IA_PD identified
+    /// by specified IAID.
+    ///
+    /// @param prefix Prefix.
+    /// @param prefix_len Prefix length.
+    /// @param iaid IAID of the IA_PD in which the lease is expected.
+    bool hasLeaseForPrefix(const asiolink::IOAddress& prefix,
+                           const uint8_t prefix_len,
+                           const IAID& iaid) const;
+
+    /// @brief Checks if client has a lease belonging to a prefix pool.
+    ///
+    /// @param prefix Pool prefix.
+    /// @param prefix_len Prefix length.
+    /// @param delegated_len Delegated prefix length.
+    ///
+    /// @return true if client has a lease belonging to specified pool,
+    /// false otherwise.
+    bool hasLeaseForPrefixPool(const asiolink::IOAddress& prefix,
+                               const uint8_t prefix_len,
+                               const uint8_t delegated_len) const;
+
+    /// @brief Checks if client has a lease with zero lifetimes for a prefix.
+    ///
+    /// @param prefix Prefix.
+    /// @param prefix_len Prefix length.
+    ///
+    /// @return true if client has a lease with zero lifetimes for a prefix.
+    bool hasLeaseWithZeroLifetimeForPrefix(const asiolink::IOAddress& prefix,
+                                           const uint8_t prefix_len) const;
+
     /// @brief Returns the value of the global status code for the last
     /// transaction.
     uint16_t getStatusCode() const {
@@ -420,67 +544,68 @@ public:
         iface_name_ = iface_name;
     }
 
-    /// @brief Set an address hint to be sent to a server.
+    /// @brief Sets link local address used by the client.
     ///
-    /// @param pref_lft Preferred lifetime.
-    /// @param valid_lft Valid lifetime.
-    /// @param address Address for which the client has a preference.
-    void useHint(const uint32_t pref_lft, const uint32_t valid_lft,
-                 const std::string& address);
-
-    /// @brief Sets a prefix hint to be sent to a server.
-    ///
-    /// @param pref_lft Preferred lifetime.
-    /// @param valid_lft Valid lifetime.
-    /// @param len Prefix length.
-    /// @param prefix Prefix for which the client has a preference.
-    void useHint(const uint32_t pref_lft, const uint32_t valid_lft,
-                 const uint8_t len, const std::string& prefix);
-
-    /// @brief Place IA_NA options to request address assignment.
-    ///
-    /// This function configures the client to place IA_NA options in its
-    /// Solicit messages to request the IPv6 address assignment.
-    ///
-    /// @param iaid IAID to be used in the IA_NA.
-    void useNA(const uint32_t iaid) {
-        useNA(true, iaid);
+    /// @param link_local New link local address.
+    void setLinkLocal(const asiolink::IOAddress& link_local) {
+        link_local_ = link_local;
     }
 
-    /// @brief Place IA_NA options to request address assignment.
+    /// @brief Specifies address to be included in client's message.
     ///
-    /// This function configures the client to place IA_NA options in its
-    /// Solicit messages to request the IPv6 address assignment.
+    /// This method specifies IPv6 address to be included within IA_NA
+    /// option sent by the client. In order to specify multiple addresses
+    /// to be included in a particular IA_NA, this method must be called
+    /// multiple times to specify each address separately. In such case,
+    /// the value of the IAID should remain the same across all calls to
+    /// this method.
     ///
-    /// @param use Parameter which 'true' value indicates that client should
-    /// request address assignment.
-    /// @param iaid IAID to be used in the IA_NA.
-    void useNA(const bool use = true, const uint32_t iaid = 1234) {
-        use_na_ = use;
-        na_iaid_ = iaid;
-    }
+    /// This method is typically called to specify IA_NA options to be
+    /// sent to the server during 4-way handshakes and during lease
+    /// renewal to request allocation of new leases (as per RFC7550).
+    ///
+    /// @param iaid IAID.
+    /// @param address IPv6 address to be included in the IA_NA. It defaults
+    /// to IPv6 zero address, which indicates that no address should be
+    /// included in the IA_NA (empty IA_NA will be sent).
+    void requestAddress(const uint32_t iaid = 1234,
+                        const asiolink::IOAddress& address =
+                        asiolink::IOAddress::IPV6_ZERO_ADDRESS());
 
-    /// @brief Place IA_PD options to request address assignment.
+    /// @brief Specifies IPv6 prefix to be included in client's message.
     ///
-    /// This function configures the client to place IA_NA options in its
-    /// Solicit messages to request the IPv6 address assignment.
+    /// This method specifies IPv6 prefix to be included within IA_PD
+    /// option sent by the client. In order to specify multiple prefixes
+    /// to be included in a particular IA_PD, this method must be called
+    /// multiple times to specify each prefix separately. In such case,
+    /// the value of the IAID should remain the same across all calls to
+    /// this method.
     ///
-    /// @param iaid IAID to be used in the IA_PD.
-    void usePD(const uint32_t iaid) {
-        usePD(true, iaid);
-    }
+    /// This method is typically called to specify IA_PD options to be
+    /// sent to the server during 4-way handshakes and during lease
+    /// renewal to request allocation of new leases (as per RFC7550).
+    ///
+    /// @param iaid IAID.
+    /// @param prefix_len Prefix length.
+    /// @param prefix Prefix to be included. This value defaults to the
+    /// IPv6 zero address. If zero address is specified and prefix_len is
+    /// set to 0, the IA Prefix option will not be included in the IA_PD.
+    /// If the prefix_len is non-zero and the prefix is IPv6 zero address
+    /// the prefix length hint will be included in the IA Prefix option.
+    void requestPrefix(const uint32_t iaid = 5678,
+                       const uint8_t prefix_len = 0,
+                       const asiolink::IOAddress& prefix =
+                       asiolink::IOAddress::IPV6_ZERO_ADDRESS());
 
-    /// @brief Place IA_PD options to request prefix assignment.
+    /// @brief Removes IAs specified by @ref requestAddress and
+    /// @ref requestPrefix methods.
     ///
-    /// This function configures the client to place IA_PD options in its
-    /// Solicit messages to request the IPv6 address assignment.
-    ///
-    /// @param use Parameter which 'true' value indicates that client should
-    /// request prefix assignment.
-    /// @param iaid IAID to be used in the IA_NA.
-    void usePD(const bool use = true, const uint32_t iaid = 5678) {
-        use_pd_ = use;
-        pd_iaid_ = iaid;
+    /// If this method is called and the client initiates an exchange with
+    /// a server the client will only include IAs for which it has leases.
+    /// If the client has no leases (e.g. a Solicit case), no IAs will be
+    /// included in the client's message.
+    void clearRequestedIAs() {
+        client_ias_.clear();
     }
 
     /// @brief Simulate sending messages through a relay.
@@ -500,14 +625,6 @@ public:
         use_client_id_ = send;
     }
 
-    /// @brief Controls whether the client should send an addres in IA_NA
-    ///
-    /// @todo: For now, this flag is only used in Decline
-    /// @param send should the address be included?
-    void includeAddress(const bool send) {
-        include_address_ = send;
-    }
-
     /// @brief Specifies if the Rapid Commit option should be included in
     /// the Solicit message.
     ///
@@ -567,6 +684,25 @@ public:
         oro_.push_back(option_code);
     }
 
+    /// @brief Controls whether the client will send DOCSIS vendor ORO
+    ///
+    /// The actual content of the ORO is specified in docsis_oro_.
+    /// It is useful to split the actual content and the ORO sending
+    /// decision, so we could test cases of sending empty ORO.
+    /// @param send controls whether ORO will be sent or not.
+    void useDocsisORO(bool send) {
+        use_docsis_oro_ = send;
+    }
+
+    /// @brief Instructs client to request specified option in DOCSIS
+    /// vendor ORO
+    ///
+    /// @param option_code client will request this option code
+    void requestDocsisOption(uint16_t option_code) {
+        use_docsis_oro_ = true;
+        docsis_oro_.push_back(option_code);
+    }
+
     /// @brief returns client-id
     /// @return client-id
     DuidPtr getDuid() const {
@@ -576,8 +712,14 @@ public:
     /// @brief Generates IA_NA based on lease information
     ///
     /// @param query generated IA_NA options will be added here
-    void
-    generateIAFromLeases(const Pkt6Ptr& query);
+    /// @param include_address should the address be included?
+    void generateIAFromLeases(const Pkt6Ptr& query,
+                              const bool include_address = true);
+
+    /// @brief Adds extra option (an option the client will always send)
+    ///
+    /// @param opt additional option to be sent
+    void addExtraOption(const OptionPtr& opt);
 
 private:
 
@@ -617,27 +759,12 @@ private:
 
     /// @brief Includes IAs to be requested.
     ///
-    /// This method checks if @c use_na_ and/or @c use_pd_ are specified and
-    /// includes appropriate IA types, if they are not already included.
+    /// This method includes IAs explicitly requested using client_ias_
     ///
     /// @param query Pointer to the client's message to which IAs should be
     /// added.
     void appendRequestedIAs(const Pkt6Ptr& query) const;
 
-    /// @brief Include IA of the specified type if it doesn't exist yet.
-    ///
-    /// This methods includes an IA option of the specific type, and
-    /// having a given IAID to the query message, if this IA hasn't
-    /// been added yet.
-    ///
-    /// @param query Pointer to the client's message to which IA should be
-    /// added.
-    /// @param ia_type One of the D6O_IA_NA or D6O_IA_PD
-    /// @param iaid IAID of the IA.
-    void conditionallyAppendRequestedIA(const Pkt6Ptr& query,
-                                        const uint8_t ia_type,
-                                        const uint32_t iaid) const;
-
     /// @brief Copy IA options from one message to another.
     ///
     /// This method copies IA_NA and IA_PD options from one message to another.
@@ -729,19 +856,15 @@ private:
     /// @brief Pointer to the server that the client is communicating with.
     boost::shared_ptr<isc::dhcp::test::NakedDhcpv6Srv> srv_;
 
-    bool use_na_;    ///< Enable address assignment.
-    bool use_pd_;    ///< Enable prefix delegation.
     bool use_relay_; ///< Enable relaying messages to the server.
 
     bool use_oro_;  ///< Conth
+    bool use_docsis_oro_;
     bool use_client_id_;
     bool use_rapid_commit_;
 
-    /// @brief Pointer to the option holding an address hint.
-    Option6IAAddrPtr address_hint_;
-
-    /// @brief Pointer to the option holding a prefix hint.
-    Option6IAPrefixPtr prefix_hint_;
+    /// @brief List holding information to be sent in client's IAs.
+    std::list<ClientIA> client_ias_;
 
     /// @brief List of options to be requested
     ///
@@ -749,22 +872,20 @@ private:
     /// to true. See @ref sendORO for details.
     std::vector<uint16_t> oro_;
 
+    /// @brief List of DOCSIS vendor options to be requested
+    ///
+    /// Content of this vector will be sent as DOCSIS vendor ORO if
+    /// use_docsis_oro_ is set to true. See @ref sendDocsisORO for details.
+    std::vector<uint16_t> docsis_oro_;
+
     /// @brief forced (Overridden) value of the server-id option (may be NULL)
     OptionPtr forced_server_id_;
 
+    /// @brief Extra options the client will send.
+    OptionCollection extra_options_;
+
     /// @brief FQDN requested by the client.
     Option6ClientFqdnPtr fqdn_;
-
-    /// @bref IAID used by the client when requesting address assignment.
-    uint32_t na_iaid_;
-    /// @brief IAID used by the client when requesting prefix delegation.
-    uint32_t pd_iaid_;
-
-    /// @brief Determines if the client will include address in the messages
-    ///        it sends.
-    ///
-    /// @todo this flag is currently supported in Decline only.
-    bool include_address_;
 };
 
 } // end of namespace isc::dhcp::test
